package cn.iocoder.yudao.module.system.service.houselocation;

import cn.iocoder.yudao.module.system.controller.admin.houselocation.vo.WareHouseLocationTypePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.houselocation.vo.WareHouseLocationTypeSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationTypeDO;
import cn.iocoder.yudao.module.system.dal.mysql.houselocation.WareHouseLocationTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.HOUSE_LOCATION_TYPE_NOT_EXISTS;

/**
 * 库位类型 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class WareHouseLocationTypeServiceImpl extends ServiceImpl<WareHouseLocationTypeMapper, WareHouseLocationTypeDO> implements WareHouseLocationTypeService {

    @Resource
    private WareHouseLocationTypeMapper houseLocationTypeMapper;

    @Override
    public Long createHouseLocationType(WareHouseLocationTypeSaveReqVO createReqVO) {
        // 插入
        WareHouseLocationTypeDO houseLocationType = BeanUtils.toBean(createReqVO, WareHouseLocationTypeDO.class);
        houseLocationTypeMapper.insert(houseLocationType);
        // 返回
        return houseLocationType.getId();
    }

    @Override
    public void updateHouseLocationType(WareHouseLocationTypeSaveReqVO updateReqVO) {
        // 校验存在
        validateHouseLocationTypeExists(updateReqVO.getId());
        // 更新
        WareHouseLocationTypeDO updateObj = BeanUtils.toBean(updateReqVO, WareHouseLocationTypeDO.class);
        houseLocationTypeMapper.updateById(updateObj);
    }

    @Override
    public void deleteHouseLocationType(Long id) {
        // 校验存在
        validateHouseLocationTypeExists(id);
        // 删除
        houseLocationTypeMapper.deleteById(id);
    }

    private void validateHouseLocationTypeExists(Long id) {
        if (houseLocationTypeMapper.selectById(id) == null) {
            throw exception(HOUSE_LOCATION_TYPE_NOT_EXISTS);
        }
    }

    @Override
    public WareHouseLocationTypeDO getHouseLocationType(Long id) {
        return houseLocationTypeMapper.selectById(id);
    }

    @Override
    public PageResult<WareHouseLocationTypeDO> getHouseLocationTypePage(WareHouseLocationTypePageReqVO pageReqVO) {
        return houseLocationTypeMapper.selectPage(pageReqVO);
    }

    /**
     * 获取所有库位类型
     * @return
     */
    @Override
    public List<WareHouseLocationTypeDO> getHouseLocationTypeList() {
        return houseLocationTypeMapper.selectList(new LambdaQueryWrapper<WareHouseLocationTypeDO>()
                .orderByDesc(WareHouseLocationTypeDO::getLocationSort));
    }

}
