package cn.iocoder.yudao.module.system.service.log;

import java.util.*;
import javax.validation.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.log.vo.RobotTaskDetailActionLogPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.log.vo.RobotTaskDetailActionLogSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.log.RobotTaskDetailActionLogDO;

/**
 * 车辆动作记录 Service 接口
 *
 * @author 陈宾顺
 */
public interface RobotTaskDetailActionLogService {

    /**
     * 创建车辆动作记录
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createTaskDetailActionLog(@Valid RobotTaskDetailActionLogSaveReqVO createReqVO);

    /**
     * 更新车辆动作记录
     *
     * @param updateReqVO 更新信息
     */
    void updateTaskDetailActionLog(@Valid RobotTaskDetailActionLogSaveReqVO updateReqVO);

    /**
     * 删除车辆动作记录
     *
     * @param id 编号
     */
    void deleteTaskDetailActionLog(Long id);

    /**
     * 获得车辆动作记录
     *
     * @param id 编号
     * @return 车辆动作记录
     */
    RobotTaskDetailActionLogDO getTaskDetailActionLog(Long id);

    /**
     * 获得车辆动作记录分页
     *
     * @param pageReqVO 分页查询
     * @return 车辆动作记录分页
     */
    PageResult<RobotTaskDetailActionLogDO> getTaskDetailActionLogPage(RobotTaskDetailActionLogPageReqVO pageReqVO);

    /**
     * 设置上一条数据完成（非主任务）
     * @param robotNo
     * @return
     */
    RobotTaskDetailActionLogDO setPreviousTaskDoneByOrderId(Long orderId);

    /**
     * 获取车辆的最后一条任务(限处理中的任务使用)
     * @param robotNo
     * @return
     */
    RobotTaskDetailActionLogDO getLastTaskByRobotNo(String robotNo, Integer commandId);

    /**
     * 根据任务id查询
     * @param taskDetailId
     */
    List<RobotTaskDetailActionLogDO> getTaskDetailActionLogByDetailId(Long taskDetailId);

    /**
     * 批量更新
     * @param taskDetailActionLog
     */
    void updateTaskDetailActionLogs(List<RobotTaskDetailActionLogDO> taskDetailActionLog);

    /**
     * 获取未统计过的任务操作日志
     * @return
     */
    List<RobotTaskDetailActionLogDO> getUnStatisticsDurationLog();

    /**
     * 获取这台车最后一次执行的任务(查真实的分配记录)
     * @param robotNo
     * @return
     */
    RobotTaskDetailActionLogDO getOriginalLastTaskByRobotNo(String robotNo);

    void alreadyCounted(List<Long> logIds);

    RobotTaskDetailActionLogDO getLastActionLogByDetailId(Long taskDetailId);
}